;===============================;
;V Gundam: VWF			;
;===============================;
;Assembled with xkas v0.06	;
;===============================;
;JSL from 84/825E - $2025e	;
;===============================;

!shift = $7f3fa0
!text_pos = $167A

!char = $1684
!ram_pos = $167c

!tiles = $868000
!width_table = $86b000

lorom
org $869400			;$31400

start:
	rep #$30

check_shift:
	lda !shift
	and #$00ff
	bne not_empty		;there's still space left on the current tile

new_tile:
	jsr tile_to_buffer	;load the tile into the vwf buffer
	jsr buffer_to_dma	;copy it into ram for later
	jsr set_width
	rtl

not_empty:
	cmp #$0008		;this chunk handles the ram offset
	beq tile_full		;full but not spilling into the 2nd 8x16
	bcs tile_overflow	;full and spilling over
	bra fill_tile		;not full yet

tile_full:
	sec
	sbc #$0008
	sta !shift

	inc !ram_pos		;no need to merge the buffers since they were
	bra new_tile		;at the end of the previous iteration, so we
				;start filling the buffer again with the current
				;char

tile_overflow:
	sec
	sbc #$0008
	sta !shift

	inc !ram_pos

	sep #$20
	ldx #$0000

swap_top:
	lda $7f3f88,x		;move the second byte
	sta $7f3f80,x

	lda #$00
	sta $7f3f88,x
	inx

	cpx #$0008
	bne swap_top

	ldx #$0000

swap_bottom:
	lda $7f3f98,x		;move the second byte
	sta $7f3f90,x

	lda #$00
	sta $7f3f98,x
	inx

	cpx #$0008
	bne swap_bottom		;after this we need to chuck char into ram

fill_tile:
	jsr tile_to_buffer
	jsr vwf
	jsr buffer_to_dma	;merge the tiles so nothing is lost on
	jsr set_width		;subsequent iterations
	rtl


;===============================;
;set_width			;
;===============================;
;Sets up the shift variable for	;
;the next character.		;
;===============================;


set_width:
	rep #$30

	lda !char
	cmp #$0020
	bcc get_width		;this is a very crappy, inefficent way of
				;getting the position in the width table
_20:
	cmp #$0040
	bcs _40

	and #$000f
	ora #$0010

	bra get_width

_40:
	cmp #$0060
	bcs _60

	and #$000f
	ora #$0020

	bra get_width

_60:
	cmp #$0080
	bcs _80

	and #$000f
	ora #$0030

	bra get_width

_80:
	cmp #$00a0
	bcs _a0

	and #$000f
	ora #$0040

	bra get_width

_a0:
	cmp #$00c0
	bcs _c0

	and #$000f
	ora #$0050

	bra get_width

_c0:
	cmp #$00e0
	bcs _e0

	and #$000f
	ora #$0060

	bra get_width

_e0:
	cmp #$0100
	bcs _100

	and #$000f
	ora #$0070

	bra get_width

_100:
	cmp #$0120
	bcs _120

	and #$000f
	ora #$0080

	bra get_width

_120:
	and #$000f
	ora #$0090
	
get_width:
	tax			;widths are 2 bytes to avoid setting and resetting
				;the accumulator in the other routines

	lda !width_table,x
	inc			;account for a 1 pixel space between each character
	
	clc
	adc !shift
	sta !shift

	rts

;===============================;
;tile_to_buffer			;
;===============================;
;Stores the char specified by	;
;$1684 into the vwf buffer and	;
;clears out the overflow buffer	;
;to prevent problems when ORing	;
;and merging the data.		;
;===============================;

tile_to_buffer:
	rep #$30
	stz $00

	ldx #$8000		;2bpp font starts at $86:8000
	stx $02
	ldx #$0086
	stx $04

	lda !char		;calculate the char's position
	asl #4
	tay

	ldx #$0000
	sep #$20

top:
	iny			;ignore plane 1

	lda [$02],y
	sta $7f3f80,x		;actual tile

	iny
	inx

	cpx #$0010
	bne top

calc_bottom_pos:
	rep #$20

	tya

	clc			;top and bottom are seperate by $100 bytes
	adc #$00e0

	tay

	sep #$20
	ldx #$0000

bottom:
	iny			;ignore plane 1

	lda [$02],y
	sta $7f3f90,x

	iny
	inx

	cpx #$0010
	bne bottom

	rts

;===============================;
;buffer_to_dma			;
;===============================;
;Transfers the 8x16 tile to its	;
;proper place in ram so the	;
;original system handles all of	;
;the dma stuff for us.		;
;				;
;This transfer is accomplished	;
;by way of ORing the buffers	;
;together so that it may be used;
;for both complete and		;
;incomplete VWF tiles.		;
;				;
;The 8x16 tile is converted to	;
;2bpp on the fly.		;
;===============================;

buffer_to_dma:
	rep #$30

	ldx #$3f80		;access the vwf buffer via indirect
	stx $02			;indexed addressing

	ldx #$007f
	stx $04

	ldy #$0000
	stz $00

	lda !ram_pos
	asl #4
	tax

	sep #$20		;8-bit accumulator

merge_top:
	lda $7f4001,x		;top-left
	ora [$02],y
	sta $7f4001,x

	lda #$00
	sta $7f4000,x		;plane 1
	sta $7f4010,x		;top-right is empty
	sta $7f4011,x

	iny
	inx #2

	inc $00
	lda $00

	cmp #$10
	bcc merge_top

	rep #$20

	txa			;calculate the position of the bottom half
	clc
	adc #$01e0
	tax

	stz $00			;reset counter

	sep #$20

merge_bottom:
	lda $7f4001,x
	ora [$02],y
	sta $7f4001,x

	lda #$00
	sta $7f4000,x
	sta $7f4010,x
	sta $7f4010,x

	iny
	inx #2

	inc $00
	lda $00

	cmp #$10
	bcc merge_bottom

	rts

;===============================;
;vwf				;
;===============================;
;This does all of the shifting	;
;for the VWF. In the interest	;
;of keeping the source legible,	;
;I have separated this into its	;
;own routine.			;
;===============================;

vwf:
	rep #$30

	lda !shift
	and #$000f
	tay

	ldx #$0000
	sep #$20

shift_top:
	phy

rotate_top:
	clc

	lda $7f3f80,x		;cheap way of getting around the fact you
	ror a			;can't ror $7f3f80,x
	sta $7f3f80,x

	lda $7f3f88,x		;catch the spillover on the next tile
	ror a
	sta $7f3f88,x

	dey
	bne rotate_top

	ply
	inx

	cpx #$0008
	bne shift_top

	ldx #$0000

shift_bottom:
	phy

rotate_bottom:
	clc

	lda $7f3f90,x
	ror a
	sta $7f3f90,x

	lda $7f3f98,x
	ror a
	sta $7f3f98,x

	dey
	bne rotate_bottom

	ply
	inx

	cpx #$0008
	bne shift_bottom

	rts

org $848278

kill_ram:
	nop #6			;keep the original systems from altering
				;the ram position

;===============================;
;VWF Prep-work			;
;===============================;

;$84/81B0 jsl init_vwf - $201b0

;$84/81B0 22 3B 87 84 JSL $84873B[$84:873B]   A:0000 X:0008 Y:00B6 D:0000 DB:80 S:1FD6 P:envmxdIzcHC:1186 VC:253 00 FL:10
;$84/873B C2 20       REP #$20                A:0000 X:0008 Y:00B6 D:0000 DB:80 S:1FD3 P:envmxdIzcHC:1240 VC:253 00 FL:11
;$84/873D AD 7E 16    LDA $167E  [$80:167E]   A:0000 X:0008 Y:00B6 D:0000 DB:80 S:1FD3 P:envmxdIzcHC:1264 VC:253 00 FL:11
;$84/8740 0A          ASL A                   A:0001 X:0008 Y:00B6 D:0000 DB:80 S:1FD3 P:envmxdIzcHC:1304 VC:253 00 FL:11
;$84/8741 AA          TAX                     A:0002 X:0008 Y:00B6 D:0000 DB:80 S:1FD3 P:envmxdIzcHC:1322 VC:253 00 FL:11
;$84/8742 BF 09 BF A5 LDA $A5BF09,x[$A5:BF0B] A:0002 X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:envmxdIzcHC:1340 VC:253 00 FL:11
;$84/8746 8D 70 16    STA $1670  [$80:1670]   A:C0EF X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:eNvmxdIzcHC:0014 VC:254 00 FL:11
;$84/8749 E2 20       SEP #$20                A:C0EF X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:eNvmxdIzcHC:0054 VC:254 00 FL:11
;$84/874B A9 A5       LDA #$A5                A:C0EF X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:eNvMxdIzcHC:0078 VC:254 00 FL:11
;$84/874D 8D 72 16    STA $1672  [$80:1672]   A:C0A5 X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:eNvMxdIzcHC:0096 VC:254 00 FL:11
;$84/8750 C2 20       REP #$20                A:C0A5 X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:eNvMxdIzcHC:0128 VC:254 00 FL:11
;$84/8752 6B          RTL                     A:C0A5 X:0002 Y:00B6 D:0000 DB:80 S:1FD3 P:eNvmxdIzcHC:0152 VC:254 00 FL:11

org $8481b0

	jsl $869600

org $869600				;$31600

init_vwf:
	rep #$30

	ldx #$0000
	lda #$0000

clear_buffer:
	sta $7f3f80,x			;work buffer
	inx #2

	cpx #$0020
	bne clear_buffer

	sta $7f3fa0			;shift variable

	jsl $84873b			;set up the pointer
	rtl				;return to the calling function

;eos:
;$84/824F C9 F1 FF    CMP #$FFF1              A:FFF1 X:00A5 Y:0016 D:0000 DB:80 S:1FD6 P:eNvmxdIzcHC:1008 VC:240 00 FL:10
;$84/8252 F0 35       BEQ $35    [$8289]      A:FFF1 X:00A5 Y:0016 D:0000 DB:80 S:1FD6 P:envmxdIZCHC:1032 VC:240 00 FL:10
;$84/8289 EE 6C 16    INC $166C  [$80:166C]   A:FFF1 X:00A5 Y:0016 D:0000 DB:80 S:1FD6 P:envmxdIZCHC:1056 VC:240 00 FL:10
;$84/828C A2 10 00    LDX #$0010              A:FFF1 X:00A5 Y:0016 D:0000 DB:80 S:1FD6 P:envmxdIzCHC:1118 VC:240 00 FL:10
;$84/828F 8E 88 16    STX $1688  [$80:1688]   A:FFF1 X:0010 Y:0016 D:0000 DB:80 S:1FD6 P:envmxdIzCHC:1142 VC:240 00 FL:10
;$84/8292 9C 8A 16    STZ $168A  [$80:168A]   A:FFF1 X:0010 Y:0016 D:0000 DB:80 S:1FD6 P:envmxdIzCHC:1182 VC:240 00 FL:10
;$84/8295 60          RTS                     A:FFF1 X:0010 Y:0016 D:0000 DB:80 S:1FD6 P:envmxdIzCHC:1222 VC:240 00 FL:10

;line_break:
;$84/8254 C9 F0 FF    CMP #$FFF0              A:FFF0 X:00A5 Y:003C D:0000 DB:80 S:1FD5 P:eNvmxdIzcHC:1052 VC:240 00 FL:10
;$84/8257 F0 3D       BEQ $3D    [$8296]      A:FFF0 X:00A5 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIZCHC:1076 VC:240 00 FL:10
;$84/8296 22 23 85 84 JSL $848523[$84:8523]   A:FFF0 X:00A5 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIZCHC:1100 VC:240 00 FL:10
;$84/8523 A5 A6       LDA $A6    [$00:00A6]   A:FFF0 X:00A5 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIZCHC:1154 VC:240 00 FL:11
;$84/8525 29 80 00    AND #$0080              A:0000 X:00A5 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIZCHC:1188 VC:240 00 FL:11
;$84/8528 D0 07       BNE $07    [$8531]      A:0000 X:00A5 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIZCHC:1212 VC:240 00 FL:11
;$84/852A A2 08 00    LDX #$0008              A:0000 X:00A5 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIZCHC:1230 VC:240 00 FL:11
;$84/852D 8E 80 16    STX $1680  [$80:1680]   A:0000 X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:1254 VC:240 00 FL:11
;$84/8530 6B          RTL                     A:0000 X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:1294 VC:240 00 FL:11
;$84/829A EE 7A 16    INC $167A  [$80:167A]   A:0000 X:0008 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIzCHC:1342 VC:240 00 FL:10
;$84/829D AD 7C 16    LDA $167C  [$80:167C]   A:0000 X:0008 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0036 VC:241 00 FL:10
;$84/82A0 C9 1D 00    CMP #$001D              A:005A X:0008 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0076 VC:241 00 FL:10
;$84/82A3 B0 05       BCS $05    [$82AA]      A:005A X:0008 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0100 VC:241 00 FL:10
;$84/82AA 22 33 86 84 JSL $848633[$84:8633]   A:005A X:0008 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0124 VC:241 00 FL:10
;$84/8633 C2 20       REP #$20                A:005A X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:0178 VC:241 00 FL:11
;$84/8635 AD 7E 16    LDA $167E  [$80:167E]   A:005A X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:0202 VC:241 00 FL:11
;$84/8638 29 00 80    AND #$8000              A:0001 X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:0242 VC:241 00 FL:11
;$84/863B D0 08       BNE $08    [$8645]      A:0000 X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIZCHC:0266 VC:241 00 FL:11
;$84/863D A2 80 00    LDX #$0080              A:0000 X:0008 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIZCHC:0284 VC:241 00 FL:11
;$84/8640 8E 7C 16    STX $167C  [$80:167C]   A:0000 X:0080 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:0308 VC:241 00 FL:11
;$84/8643 80 06       BRA $06    [$864B]      A:0000 X:0080 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:0348 VC:241 00 FL:11
;$84/864B 6B          RTL                     A:0000 X:0080 Y:003C D:0000 DB:80 S:1FD2 P:envmxdIzCHC:0372 VC:241 00 FL:11
;$84/82AE 60          RTS                     A:0000 X:0080 Y:003C D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0420 VC:241 00 FL:10
;$84/816B 6B          RTL                     A:0000 X:0080 Y:003C D:0000 DB:80 S:1FD7 P:envmxdIzCHC:0466 VC:241 00 FL:09

org $848296

	jsl $86a000

org $86a000

line_break:
	rep #$30

	ldx #$0000			;reset the variables for the next line
	lda #$0000

reset_buff:
	sta $7f3f80,x			;work buffer
	inx #2

	cpx #$0020
	bne reset_buff

	sta $7f3fa0			;shift variable

	jsl $848523			;the original routine that was jsl'd to
	rtl


;tells the game to write text in the top half ("Zanscare Area")
;$84/824A C9 FF FF    CMP #$FFFF              A:FFFF X:00A5 Y:0052 D:0000 DB:80 S:1FD5 P:eNvmxdIzcHC:0604 VC:241 00 FL:10
;$84/824D F0 34       BEQ $34    [$8283]      A:FFFF X:00A5 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIZCHC:0628 VC:241 00 FL:10
;$84/8283 A2 01 00    LDX #$0001              A:FFFF X:00A5 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIZCHC:0652 VC:241 00 FL:10
;$84/8286 8E C4 16    STX $16C4  [$80:16C4]   A:FFFF X:0001 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0676 VC:241 00 FL:10
;$84/8289 EE 6C 16    INC $166C  [$80:166C]   A:FFFF X:0001 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0716 VC:241 00 FL:10
;$84/828C A2 10 00    LDX #$0010              A:FFFF X:0001 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0778 VC:241 00 FL:10
;$84/828F 8E 88 16    STX $1688  [$80:1688]   A:FFFF X:0010 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0802 VC:241 00 FL:10
;$84/8292 9C 8A 16    STZ $168A  [$80:168A]   A:FFFF X:0010 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0842 VC:241 00 FL:10
;$84/8295 60          RTS                     A:FFFF X:0010 Y:0052 D:0000 DB:80 S:1FD5 P:envmxdIzCHC:0882 VC:241 00 FL:10
;$84/816B 6B          RTL                     A:FFFF X:0010 Y:0052 D:0000 DB:80 S:1FD7 P:envmxdIzCHC:0928 VC:241 00 FL:09

;===============================;
;VWF Width Table		;
;===============================;

org $86b000			;$33000

space:
	db $04,$00
down_arrow:
	db $08,$00
0:
	db $04,$00
1:
	db $03,$00
2:
	db $04,$00
3:
	db $04,$00
4:
	db $04,$00
5:
	db $04,$00
6:
	db $04,$00
7:
	db $04,$00
8:
	db $04,$00
9:
	db $04,$00
A:
	db $05,$00
B:
	db $04,$00
C:
	db $04,$00
D:
	db $04,$00
E:
	db $04,$00
F:
	db $04,$00
G:
	db $04,$00
H:
	db $04,$00
I:
	db $03,$00
J:
	db $04,$00
K:
	db $05,$00
L:
	db $04,$00
M:
	db $05,$00
N:
	db $05,$00
O:
	db $04,$00
P:
	db $04,$00
Q:
	db $04,$00
R:
	db $04,$00
S:
	db $04,$00
T:
	db $05,$00
U:
	db $04,$00
V:
	db $05,$00
W:
	db $05,$00
X:
	db $05,$00
Y:
	db $05,$00
Z:
	db $04,$00
a:
	db $04,$00
b:
	db $04,$00
c:
	db $04,$00
d:
	db $04,$00
e:
	db $04,$00
f:
	db $04,$00
g:
	db $04,$00
h:
	db $04,$00
i:
	db $01,$00
j:
	db $03,$00
k:
	db $05,$00
l:
	db $02,$00
m:
	db $05,$00
n:
	db $04,$00
o:
	db $04,$00
p:
	db $04,$00
q:
	db $04,$00
r:
	db $04,$00
s:
	db $04,$00
t:
	db $03,$00
u:
	db $04,$00
v:
	db $05,$00
w:
	db $05,$00
x:
	db $05,$00
y:
	db $05,$00
z:
	db $04,$00
@:
	db $08,$00
roman_one:
	db $04,$00
roman_two:
	db $07,$00
,:
	db $02,$00
.:
	db $01,$00
':
	db $02,$00
question_mark:
	db $04,$00
exclamation:
	db $01,$00
..:
	db $03,$00
[:
	db $05,$00
]:
	db $05,$00
~:
	db $07,$00
-:
	db $03,$00
/:
	db $04,$00
::
	db $01,$00
*:
	db $08,$00
